@@ -282,19 +282,23 @@ static void S_make_instance_fields(pTHX_ const ClassMeta *classmeta, AV *backing
     FieldMeta *fieldmeta = (FieldMeta *)AvARRAY(fields)[i];
     char sigil = SvPV_nolen(fieldmeta->name)[0];
 
-    assert(av_count(backingav) == fieldmeta->fieldix + roleoffset);
+    FIELDOFFSET fieldix = fieldmeta->fieldix + roleoffset;
+
+    /* We can't av_push() because REPR_KEYS would break here */
+    SV **svp = av_fetch_simple(backingav, fieldix, TRUE);
+    assert(svp);
 
     switch(sigil) {
       case '$':
-        av_push(backingav, newSV(0));
+        /* simply fetching will create the SV */
         break;
 
       case '@':
-        av_push(backingav, newRV_noinc((SV *)newAV()));
+        sv_setrv_noinc(*svp, (SV *)newAV());
         break;
 
       case '%':
-        av_push(backingav, newRV_noinc((SV *)newHV()));
+        sv_setrv_noinc(*svp, (SV *)newHV());
         break;
 
       default:
@@ -319,6 +323,27 @@ static void S_make_instance_fields(pTHX_ const ClassMeta *classmeta, AV *backing
   }
 }
 
+#define alias_fieldkeys_into_av(classmeta, hv, backingav)  S_alias_fieldkeys_into_av(aTHX_ classmeta, hv, backingav)
+static void S_alias_fieldkeys_into_av(pTHX_ ClassMeta *classmeta, HV *hv, AV *backingav)
+{
+  if(classmeta->cls.supermeta)
+    alias_fieldkeys_into_av(classmeta->cls.supermeta, hv, backingav);
+
+  AV *fields = classmeta->direct_fields;
+  I32 nfields = av_count(fields);
+
+  I32 i;
+  for(i = 0; i < nfields; i++) {
+    FieldMeta *fieldmeta = (FieldMeta *)AvARRAY(fields)[i];
+
+    SV *fieldkey = newSVpvf("%" SVf "/%" SVf, classmeta->name, fieldmeta->name);
+    HE *he = hv_fetch_ent(hv, fieldkey, 1, 0);
+    SvREFCNT_dec(fieldkey);
+
+    av_store(backingav, fieldmeta->fieldix, SvREFCNT_inc(HeVAL(he)));
+  }
+}
+
 SV *ObjectPad_get_obj_backingav(pTHX_ SV *self, enum ReprType repr, bool create)
 {
   SV *rv = SvRV(self);
@@ -373,6 +398,19 @@ SV *ObjectPad_get_obj_backingav(pTHX_ SV *self, enum ReprType repr, bool create)
       if(SvTYPE(rv) == SVt_PVHV)
         goto case_REPR_HASH;
       goto case_REPR_MAGIC;
+
+    case REPR_KEYS:
+    {
+      /* TODO: This representation is going to be sloooooow
+       */
+      if(SvTYPE(rv) != SVt_PVHV)
+        croak("Not a HASH reference");
+      HV *hv = (HV *)rv;
+      AV *backingav = newAV();
+      SAVEFREESV((SV *)backingav);
+      alias_fieldkeys_into_av(mop_get_class_for_stash(SvSTASH(rv)), hv, backingav);
+      return (SV *)backingav;
+    }
   }
 
   croak("ARGH unhandled repr type");
@@ -1243,17 +1281,13 @@ static void S_generate_initfields_method(pTHX_ ClassMeta *meta)
     ops = op_append_list(OP_LINESEQ, ops,
       newSTATEOP(0, NULL, NULL));
 
-    /* Build an OP_ENTERSUB for supermeta's initfields */
-    OP *op = NULL;
-    op = op_append_list(OP_LIST, op,
-      newPADxVOP(OP_PADSV, 0, PADIX_SELF));
-    op = op_append_list(OP_LIST, op,
-      newPADxVOP(OP_PADHV, OPf_REF, PADIX_PARAMS));
-    op = op_append_list(OP_LIST, op,
-      newSVOP(OP_CONST, 0, (SV *)supermeta->initfields));
-
     ops = op_append_list(OP_LINESEQ, ops,
-      op_convert_list(OP_ENTERSUB, OPf_WANT_VOID|OPf_STACKED, op));
+      /* Build an OP_ENTERSUB for supermeta's initfields */
+      newLISTOPn(OP_ENTERSUB, OPf_WANT_VOID|OPf_STACKED,
+        newPADxVOP(OP_PADSV, 0, PADIX_SELF),
+        newPADxVOP(OP_PADHV, OPf_REF, PADIX_PARAMS),
+        newSVOP(OP_CONST, 0, (SV *)supermeta->initfields),
+        NULL));
   }
 
   if(meta->initfields_lines) {
@@ -1280,16 +1314,12 @@ static void S_generate_initfields_method(pTHX_ ClassMeta *meta)
       ops = op_append_list(OP_LINESEQ, ops,
         newSTATEOP(0, NULL, NULL));
 
-      OP *op = NULL;
-      op = op_append_list(OP_LIST, op,
-        newPADxVOP(OP_PADSV, 0, PADIX_SELF));
-      op = op_append_list(OP_LIST, op,
-        newPADxVOP(OP_PADHV, OPf_REF, PADIX_PARAMS));
-      op = op_append_list(OP_LIST, op,
-        newSVOP(OP_CONST, 0, (SV *)embed_cv(rolemeta->initfields, embedding)));
-
       ops = op_append_list(OP_LINESEQ, ops,
-        op_convert_list(OP_ENTERSUB, OPf_WANT_VOID|OPf_STACKED, op));
+        newLISTOPn(OP_ENTERSUB, OPf_WANT_VOID|OPf_STACKED,
+          newPADxVOP(OP_PADSV, 0, PADIX_SELF),
+          newPADxVOP(OP_PADHV, OPf_REF, PADIX_PARAMS),
+          newSVOP(OP_CONST, 0, (SV *)embed_cv(rolemeta->initfields, embedding)),
+          NULL));
     }
   }
 
@@ -1343,6 +1373,10 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
       croak("Class %" SVf " does not provide a required method named '%" SVf "'",
         SVfARG(meta->name), SVfARG(mname));
     }
+
+    GV *gv = gv_fetchmeth_pvs(meta->stash, "BUILDARGS", -1, 0);
+    if(GvSTASH(gv) != gv_stashpvs("Object::Pad::UNIVERSAL", 0))
+      meta->has_buildargs = true;
   }
 
   if(meta->strict_params && meta->buildblocks)
@@ -1441,9 +1475,10 @@ XS_INTERNAL(injected_constructor)
    * This does not include $self
    */
   AV *args = newAV();
+  I32 nargs = 0;
   SAVEFREESV(args);
 
-  {
+  if(meta->has_buildargs) {
     /* @args = $class->BUILDARGS(@_) */
     ENTER;
     SAVETMPS;
@@ -1467,16 +1502,23 @@ XS_INTERNAL(injected_constructor)
     SP++;
     PUTBACK;
 
-    I32 nargs = call_method("BUILDARGS", G_ARRAY);
+    nargs = call_method("BUILDARGS", G_ARRAY);
 
     SPAGAIN;
 
     for(svp = SP - nargs + 1; svp <= SP; svp++)
-      av_push(args, SvREFCNT_inc(*svp));
+      av_push_simple(args, SvREFCNT_inc(*svp));
 
     FREETMPS;
     LEAVE;
   }
+  else {
+    nargs = items - 1;
+
+    SV **svp;
+    for(svp = SP - nargs + 1; svp <= SP; svp++)
+      av_push_simple(args, SvREFCNT_inc(*svp));
+  }
 
   bool need_makefields = true;
 
@@ -1494,6 +1536,7 @@ XS_INTERNAL(injected_constructor)
         break;
 
       case REPR_HASH:
+      case REPR_KEYS:
         DEBUG_SET_CURCOP_LINE(__LINE__);
         self = sv_2mortal(newRV_noinc((SV *)newHV()));
         sv_bless(self, stash);
@@ -1512,9 +1555,9 @@ XS_INTERNAL(injected_constructor)
       SAVETMPS;
 
       PUSHMARK(SP);
-      EXTEND(SP, 1 + AvFILL(args));
+      EXTEND(SP, nargs);
 
-      SV **argstart = SP - AvFILL(args) - 1;
+      SV **argstart = SP - nargs;
       SV **argtop = SP;
       SV **svp;
 
@@ -1556,6 +1599,7 @@ XS_INTERNAL(injected_constructor)
 
       case REPR_HASH:
       case_REPR_HASH:
+      case REPR_KEYS:
         if(SvTYPE(rv) != SVt_PVHV) {
 #ifdef DEBUG_OVERRIDE_PLCURCOP
           PL_curcop = prevcop;
@@ -1625,16 +1669,16 @@ XS_INTERNAL(injected_constructor)
     paramhv = newHV();
     SAVEFREESV((SV *)paramhv);
 
-    if(av_count(args) % 2)
+    if(nargs % 2)
       warn("Odd-length list passed to %" SVf " constructor", class);
 
     /* TODO: I'm sure there's an newHV_from_AV() around somewhere */
     SV **argsv = AvARRAY(args);
 
     IV idx;
-    for(idx = 0; idx < av_count(args); idx += 2) {
+    for(idx = 0; idx < nargs; idx += 2) {
       SV *name  = argsv[idx];
-      SV *value = idx < av_count(args)-1 ? argsv[idx+1] : &PL_sv_undef;
+      SV *value = idx < nargs-1 ? argsv[idx+1] : &PL_sv_undef;
 
       hv_store_ent(paramhv, name, SvREFCNT_inc(value), 0);
     }
@@ -1676,14 +1720,14 @@ XS_INTERNAL(injected_constructor)
       SAVETMPS;
       SPAGAIN;
 
-      EXTEND(SP, 1 + AvFILL(args));
+      EXTEND(SP, nargs);
 
       PUSHMARK(SP);
 
       PUSHs(self);
 
       int argi;
-      for(argi = 0; argi <= AvFILL(args); argi++)
+      for(argi = 0; argi < nargs; argi++)
         PUSHs(argsvs[argi]);
       PUTBACK;
 
@@ -2232,6 +2276,8 @@ static bool classhook_repr_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **att
       croak("Cannot switch to :repr(magic) without a foreign superclass");
     classmeta->repr = REPR_MAGIC;
   }
+  else if(strEQ(val, "keys"))
+    classmeta->repr = REPR_KEYS;
   else if(strEQ(val, "default") || strEQ(val, "autoselect"))
     classmeta->repr = REPR_AUTOSELECT;
   else
